@@ -1,5 +1,5 @@
 # $File: //member/autrijus/cpanplus/dist/lib/CPANPLUS/Configure/Setup.pm $
-# $Revision: #6 $ $Change: 3548 $ $DateTime: 2002/03/26 09:09:27 $
+# $Revision: #14 $ $Change: 3825 $ $DateTime: 2002/04/09 11:17:33 $
 
 ##################################################
 ###        CPANPLUS/Configure/Setup.pm         ###
@@ -9,6 +9,7 @@
 package CPANPLUS::Configure::Setup;
 
 use strict;
+use vars '$AutoSetup';
 #use Exporter;
 #use CPANPLUS::Configure;
 #our @ISA = qw(Exporter);
@@ -46,24 +47,41 @@ my $term;
 
 ## gather information needed to initialize CPANPLUS
 ##
-## (takes Configure object, returns no values)
+## (takes conf => Configure object and term => Term object, returns no values)
 ##
 sub init {
-    my $conf = shift;
+    my ($self, %args) = @_;
+
+    my $conf = $args{conf};
+    $term = $args{term} if exists $args{term};
+
     local $SIG{INT};
 
     #my ($answer, $prompt, $default);
-
     print qq[
 
 CPAN is the world-wide archive of perl resources. It consists of about
 100 sites that all replicate the same contents all around the globe.
 Many countries have at least one CPAN site already. The resources found
-on CPAN are easily accessible with the CPANPLUS.pm module. If you want
-to use CPANPLUS.pm, you have to configure it properly.
+on CPAN are easily accessible with CPANPLUS modules. If you want to use
+CPANPLUS, you have to configure it properly.
 
 ];
 
+    print qq[
+Although we recommend an interactive configuration session, you can
+also enter 'n' here to use default values for all questions.
+
+];
+
+    my $answer = _get_reply(
+        prompt  => "Are you ready for manual configuration? [Y/n]: ",
+        default => 'y',
+        choices => [ qw/y n/ ],
+    );
+
+    local $AutoSetup = 1 if ($answer =~ /^n/i);
+
     _setup_ftp($conf);
     _setup_build($conf);
     _setup_conf($conf);
@@ -100,26 +118,25 @@ sub _setup_conf {
     ################
 
     print qq[
-Every Makefile.PL is run by perl in a separate process. Likewise we
-run 'make' and 'make install' in processes. If you have any
+Every Makefile.PL is run by perl in a separate process. Likewise we run
+'make' and 'make install' as separate processes. If you have any
 parameters (e.g. PREFIX, LIB, UNINST or the like) you want to pass
 to the calls, please specify them here.
 
+In particular, 'UNINST=1' is recommended for root users, unless you have
+fine-tuned ideas of where modules should be installed in the \@INC path.
+
+Enter a name=value list separated by whitespace, but quote any embedded
+spaces that you want to preserve.  (Enter a space to clear any existing
+settings.)
+
 If you don't understand this question, just press ENTER.
 
 ];
 
-    my $makeflags = $conf->get_conf('makeflags') || '';
-
-    $makeflags = _get_reply(
-                    prompt  => "Parameters for the 'make' command? [$makeflags]: ",
-                    default => $makeflags,
-                 );
-
-    #print "\nYour 'make' flags are now:\n    $makeflags\n", if ($makeflags);
-    printf "\nYour 'make' flags are now:\n    %s\n\n", $makeflags || '*nothing entered*';
-    #print "\n";
-
+    my $makeflags = _ask_flags(
+        "'make'" => $conf->get_conf('makeflags'),
+    );
 
     #####################
     ## makemaker flags ##
@@ -132,54 +149,19 @@ private user directory, set 'makemakerflags' to:
 
 LIB=~/perl/lib INSTALLMAN1DIR=~/perl/man/man1 INSTALLMAN3DIR=~/perl/man/man3
 
-and make sure you did not set UNINST=1 in 'makeflags'. :o)
+and make sure you did not set UNINST=1 in 'makeflags' above. :o)
 
 Enter a name=value list separated by whitespace, but quote any embedded
 spaces that you want to preserve.  (Enter a space to clear any existing
 settings.)
 
+Again, if you don't understand this question, just press ENTER.
+
 ];
 
-    my $MMflags = $conf->get_conf('makemakerflags');
-
-    if (%{$MMflags}) {
-        print "Your current MakeMaker flags are:\n";
-        print map {
-            defined($MMflags->{$_})
-                ? "    $_=$MMflags->{$_}\n"
-                : "    $_\n"
-        } sort keys %{$MMflags};
-        print "\n\n";
-    } #if
-
-    $answer = _get_reply(
-                  prompt  => "Parameters for MakeMaker?: ",
-              );
-
-    if ($answer) {
-        if ($answer =~ m/^\s*$/) {
-            $MMflags = {};
-        } else {
-            %{$MMflags} = map {
-                /=/ ? split('=', $_, 2) : ($_ => undef)
-            } $answer =~ m/\s*((?:[^\s=]+=)?(?:"[^"]+"|'[^']+'|[^\s]+))/g
-        } #if
-    } #if
-
-    print "\nYour MakeMaker flags are now:\n";
-
-    if (%{$MMflags}) {
-        print map {
-            defined($MMflags->{$_})
-                ? "    $_=$MMflags->{$_}\n"
-                : "    $_\n"
-        } sort keys %{$MMflags};
-    } else {
-        print "    *nothing entered*\n";
-    } #if
-
-    print "\n";
-
+    my $MMflags = _ask_flags(
+        MakeMaker => $conf->get_conf('makemakerflags'),
+    );
 
     #################
     ## shift a lib ##
@@ -196,7 +178,7 @@ current value enter a single space.)
     my $lib = $conf->get_conf('lib');
 
     $answer = _get_reply(
-                  prompt  => "Additional \@INC directories to add?[@{$lib}]: ",
+                  prompt  => "Additional \@INC directories to add? [@{$lib}]: ",
                   default => "@{$lib}",
               );
 
@@ -358,9 +340,10 @@ The modules in the CPAN archives are protected with md5 checksums.
 
 ];
 
+    my $have_md5 = eval "use Digest::MD5; 1";
     $answer = _get_reply(
-                    prompt  => "Use the md5 checksums? [Y/n]: ",
-                    default => 'y',
+                    prompt  => "Use the md5 checksums? "._yn($have_md5),
+                    default => $have_md5 ? 'y' : 'n',
                     choices => [ qw/y n/ ],
               );
 
@@ -410,9 +393,10 @@ information.  Would you like to do this?
 
 ];
 
+    my $have_storable = eval "use Storable; 1";
     $answer = _get_reply(
-                    prompt  => "Use Storable? [Y/n]: ",
-                    default => 'y',
+                    prompt  => "Use Storable? "._yn($have_storable),
+                    default => $have_storable ? 'y' : 'n',
                     choices => [ qw/y n/ ],
               );
 
@@ -430,11 +414,45 @@ information.  Would you like to do this?
     print "\n\n";
 
 
+    ###################
+    ## use cpantest? ##
+    ###################
+
+    print q[
+CPANPLUS comes with the "cpantest" utility, which can be utilized to
+report success and failures of modules installed by CPANPLUS.  Would
+you like to do this?  Note that you will still be prompted before
+sending each report.
+
+];
+
+    $answer = _get_reply(
+                    prompt  => "Report tests results? [y/N]: ",
+                    default => 'n',
+                    choices => [ qw/y n/ ],
+              );
+
+    my $cpantest;
+    print "\nOk, ";
+
+    if ($answer =~ /^y/i) {
+        $cpantest = 1;
+        print "I will prompt you to";
+    } else {
+        $cpantest = 0;
+        print "I won't";
+    } #if
+
+    print " report test results.\n\n";
+
+    print "\n\n";
+
     ##############
     ## save it! ##
     ##############
 
     $conf->set_conf(
+        cpantest       => $cpantest,
         force          => $force,
         lib            => $lib,
         makeflags      => $makeflags,
@@ -501,28 +519,42 @@ like one.
 
 ];
 
-    my $email = $conf->_get_ftp('email') || '';
+    my $email   = $conf->_get_ftp('email') || 'cpanplus@example.com';
+    my $cf_mail = $Config{cf_email};
 
-    # default the email to perl administrator
-    $email = $Config{cf_email}
-        if ($email eq 'cpanplus@example.com' and defined $Config{cf_email});
+    $cf_mail = 'cpanplus@example.com' if $cf_mail eq $email; # for variety's sake
 
-    while ('pigs fly') {
-        $answer = _get_reply(
-                        prompt  => "Email address? [$email]: ",
-                        default => $email,
-                  );
+    print qq|
+You have several choices:
 
-        ### very simple and probably incorrect email sanity checker
-        if (_valid_email($answer)) {
-            print "You lazy hoser!  You really *should* set this properly.\n"
-                if $answer eq 'cpanplus@example.com';
-            $email = $answer;
-            last;
-        } else {
-            print "You did not enter a valid email address, please try again!\n";
-            next;
-        }
+1) $email
+2) $cf_mail
+3) something else
+
+|;
+
+    $prompt = 'Please pick one [1]: ';
+    $default = '1';
+
+    while (defined($answer = _readline($prompt))) {
+        $answer ||= $default;
+        $term->addhistory($answer);
+
+                           last, if $answer == 1;
+        $email = $cf_mail, last, if $answer == 2;
+        $email = '',       last, if $answer == 3;
+
+        $prompt  = 'Please choose 1, 2, or 3 [1]: ';
+        next;
+    } #while
+
+    until ( _valid_email($email) ) {
+        print "You did not enter a valid email address, please try again!\n"
+            if length $email;
+
+        $email = _get_reply(
+            prompt  => "Email address: ",
+        );
     } #while
 
     print "\nYour 'email' is now:\n    $email\n";
@@ -597,7 +629,7 @@ You have several choices:
         $prompt = 'Please pick one [1]: ';
         $default = '1';
 
-        while (defined($answer = $term->readline($prompt))) {
+        while (defined($answer = _readline($prompt))) {
             $answer ||= $default;
             $term->addhistory($answer);
 
@@ -636,7 +668,7 @@ First of all, I'd like to create this directory.  Where?
     } #if
 
 
-    while (defined($answer = $term->readline($prompt))) {
+    while (defined($answer = _readline($prompt))) {
         $answer ||= $default;
         $term->addhistory($answer);
 
@@ -655,7 +687,7 @@ First of all, I'd like to create this directory.  Where?
 
         if (-d $answer and not (-w _)) {
             print "I can't seem to write in this directory.\n";
-            next;
+            $AutoSetup = 0; next;
         } #if
 
         ### windoze won't make more than one dir at a time :o(
@@ -666,20 +698,20 @@ First of all, I'd like to create this directory.  Where?
             unless (-d $answer or eval { File::Path::mkpath($answer) } ) {
                 chomp($@);
                 warn "I wasn't able to create this directory.\n(The error I got was $@)\n\n";
-                next;
+                $AutoSetup = 0; next;
             } #unless
         } #scope
 
         my $autdir = File::Spec->catdir($answer, $conf->_get_build('autdir'));
         unless (-e $autdir or mkdir($autdir, 0777)) {
             warn "I wasn't able to create $autdir.\n(The error I got was $!)\n\n";
-            next; # XXX: doesn't unlink the current $answer
+            $AutoSetup = 0; next; # XXX: doesn't unlink the current $answer
         }
 
         my $moddir = File::Spec->catdir($answer, $conf->_get_build('moddir'));
         unless (-e $moddir or mkdir($moddir, 0777)) {
             warn "I wasn't able to create $moddir.\n(The error I got was $!)\n\n";
-            next; # XXX: doesn't unlink the current $answer
+            $AutoSetup = 0; next; # XXX: doesn't unlink the current $answer
         }
 
         $cpan_home = Cwd::abs_path($answer);
@@ -706,7 +738,7 @@ First of all, I'd like to create this directory.  Where?
         make     => '',
         ncftp    => '',
         ncftpget => '',
-        #pager    => '',
+        pager    => '',
         #perl     => '', # favor finding this at runtime
         #shell    => '',
         tar      => '',
@@ -719,6 +751,8 @@ First of all, I'd like to create this directory.  Where?
         #unless ($pgm eq 'perl') { # favor finding this at runtime
         my $name = $Config{$pgm} || $pgm;
 
+        $name ||= $ENV{PAGER} || 'more' if ($pgm eq 'pager');
+
         $new_name = (_find_exe($name, [@path]) || MM->maybe_command($name))
                   ? $name
                   : '';
@@ -761,7 +795,7 @@ Which '$pgm' executable should I use?
 
         } #if
 
-        while (defined($answer = $term->readline($prompt))) {
+        while (defined($answer = _readline($prompt))) {
             $answer ||= $default;
             $answer =~ s/^\s+$//;
             $term->addhistory($answer), if $answer;
@@ -778,7 +812,7 @@ Which '$pgm' executable should I use?
                 if ($answer == 3) {
                     $prompt  = "Where can I find your '$pgm' executable?: ";
                     $default = '';
-                    next;
+                    $AutoSetup = 0; next;
                 } #if
 
             } #if
@@ -791,7 +825,7 @@ Which '$pgm' executable should I use?
                 #last, unless $pgm =~ m/^make|perl$/; # favor finding this at runtime
                 last, unless $pgm eq 'make';
                 warn "Without your '$pgm' executable I can't function!\n";
-                next;
+                $AutoSetup = 0; next;
             } #unless
 
             # it better actually be a program!
@@ -803,7 +837,7 @@ Which '$pgm' executable should I use?
                 warn "I couldn't find '$pgm_name' in your PATH.\n";
                 $prompt  = "Please tell me where I can find it: ";
                 $default = '';
-                next;
+                $AutoSetup = 0; next;
             } #unless
 
             print "\nGood, I found '$pgm_name' in your PATH:\n    $answer\n";
@@ -833,16 +867,56 @@ Which '$pgm' executable should I use?
 } #_setup_build
 
 
+### helper module for makeflags and makemakerflags
+sub _ask_flags {
+    my ($name, $old) = @_;
+
+    ### do a one-level deep copy of the original value
+    my $flags = (UNIVERSAL::isa($old, 'HASH')) ? { %{$old} } : {};
+
+    if (%{$flags}) {
+        print "Your current $name flags are:\n";
+        print map {
+            defined($flags->{$_})
+                ? "    $_=$flags->{$_}\n"
+                : "    $_\n"
+        } sort keys %{$flags};
+        print "\n\n";
+    } #if
+
+    my $answer = _get_reply( prompt  => "Parameters for $name?: " );
+
+    $flags = CPANPLUS::Backend->_flags_hashref($answer);
+
+    print "\nYour $name flags are now:\n";
+
+    if (%{$flags}) {
+        print map {
+            defined($flags->{$_})
+                ? "    $_=$flags->{$_}\n"
+                : "    $_\n"
+        } sort keys %{$flags};
+    } else {
+        print "    *nothing entered*\n";
+    } #if
+
+    print "\n";
+
+    return $flags;
+}
+
+
 ## locate a given executable in the given path
 ##
 ## (takes scalar and arrayref, returns scalar)
 ##
 sub _find_exe {
-    my ($exe,$path) = @_;
+    my ($exe, $path) = @_;
+    my $param = (($exe =~ s/(\s+.*)//) ? $1 : '');
 
     for my $dir (@{$path}) {
         my $abs = File::Spec->catfile($dir, $exe);
-        return $abs, if $abs = MM->maybe_command($abs);
+        return $abs.$param if $abs = MM->maybe_command($abs);
     } #for
 
 } #_find_exe
@@ -868,6 +942,7 @@ This may take a while...
 
     unless (-e $file) {
         my $cpan = new CPANPLUS::Backend($conf) or die "can't use Backend!\n";
+
         $cpan->_fetch(
             file     => $conf->_get_source('hosts'),
             fetchdir => $conf->_get_build('base'),
@@ -990,25 +1065,23 @@ file:, ftp:, or http: -- that host a CPAN mirror.
                            prompt  => "Please choose a host [$default]: ",
                            choices => [ @{$options->{host}} ],
                            default => $default,
+                           multi   => 1,
                        );
 
             if ($pick->[0] =~ /\d/) {
-                my $host = $pick->[1];
+                for my $host (@{$pick}[1..$#{$pick}]) {
+                    if (exists $host_list->{$host}) {
+                        print "\nHost $host already selected!\n";
+                        last LOOP if $AutoSetup;
+                        next;
+                    }
 
-                #push(@{$host_list}, $host),
-                #    unless grep {/^quotemeta($host)$/} @{$host_list};
-                #my $total = scalar @{$host_list};
-
-                unless (exists $host_list->{$host}) {
-                    #push @hosts, $hosts->{$host};
                     push @hosts, $host;
                     $host_list->{$host} = $hosts->{$host};
                     my $total           = scalar(keys %{$host_list});
                     printf "\nSelected %s, %d host%s selected thus far.\n",
                         $host, $total, ($total == 1) ? '' : 's';
-                } else {
-                    print "\nHost $host already selected!\n";
-                } #unless
+                }
 
                 $next = 'host';
 
@@ -1156,6 +1229,7 @@ now.  You may enter them separately or as a space delimited list.
 ##     map_sub => subref used to display picklist
 ##     items   => arrayref with items in picklist
 ##     options => hashref with options not in picklist
+##     multi   => flag to indicate a multiple choice question
 ## and any additional args for _get_reply
 ##
 ## For your trouble you get an arrayref with the user supplied answer and the
@@ -1189,7 +1263,7 @@ sub _pick_item {
 
     ## get the reply
     my $answer = _get_reply(%args);
-    return [ $answer, $choices->{$answer} ];
+    return [ $answer, @{$choices}{split(/\s+/, $answer)} ];
 
 } #_pick_item
 
@@ -1202,6 +1276,7 @@ sub _pick_item {
 ##     prompt  => prompt for user display
 ##     default => default answer, if any
 ##     choices => list of valid replies
+##     multi   => flag to indicate a multiple choice question
 ##
 ## For your trouble you get the user supplied answer.
 ##
@@ -1212,18 +1287,18 @@ sub _get_reply {
 
     # On win32, we limit ourselves to the dumb terminal.
     # -autrijus: eventually we'd want CPANPLUS::Term that wraps this up.
-    my $TR = ($^O eq 'MSWin32') ? 'Term::ReadLine::Stub' : 'Term::ReadLine';
-
-    $term ||= $TR->new('CPAN++ Configuration', *STDIN, *STDOUT);
 
     LOOP: {
-        ### why is this better autrijus? -jmb
-        #my $answer = $term->readline($args{prompt})
-        #          || (defined($args{default}) ? $args{default} : '');
-        my $answer = $term->readline($args{prompt}) || $args{default} || '';
-        $term->addhistory($answer) if $answer;
+        my $answer = _readline($args{prompt});
+        $answer = $args{default}   unless length $answer;
+        $answer = ''               unless length $answer;
+        $term->addhistory($answer) if length $answer and !$AutoSetup;
+
         if (exists $args{choices}) {
-            unless (grep /^$answer$/i, @{$args{choices}}) {
+            my @answers = $args{multi} ? split(/\s+/, $answer) : $answer;
+            unless (@answers == grep {
+                my $ans = $_; grep { lc($_) eq lc($ans) } @{$args{choices}}
+            } @answers) {
                 #$args{prompt} = 'Invalid selection, please try again: ';
                 warn "Invalid selection, please try again.\n";
                 redo LOOP;
@@ -1234,6 +1309,17 @@ sub _get_reply {
 
 } #_get_reply
 
+sub _readline {
+    if ($AutoSetup) {
+        print @_;
+        print "\n";
+        return '';
+    }
+
+    my $TR = ($^O eq 'MSWin32') ? 'Term::ReadLine::Stub' : 'Term::ReadLine';
+    $term ||= $TR->new('CPANPLUS Configuration', *STDIN, *STDOUT);
+    return $term->readline(@_);
+}
 
 ## MIRRORED.BY parser
 ##
@@ -1372,11 +1458,11 @@ sub _guess_from_timezone {
 
     local $_;
 
-    ## pick the entry with most country/site/frequency, one level each; 
+    ## pick the entry with most country/site/frequency, one level each;
     ## note it has to be sorted -- otherwise we're depending on the hash order.
     ## also, the list context assignment (pick first one) is deliberate.
 
-    my ($continent) = map { 
+    my ($continent) = map {
         (sort { ($_->{$b} <=> $_->{$a}) or $b cmp $a } keys(%{$_}))
     } $zones{$offset};
 
@@ -1423,41 +1509,44 @@ sub _find_seq {
     my $RFC822PAT; # RFC pattern to match for valid email address
 
     sub _valid_email {
-	if (!$RFC822PAT) {
-	    my $esc        = '\\\\'; my $Period      = '\.'; my $space      = '\040';
-	    my $tab         = '\t';  my $OpenBR     = '\[';  my $CloseBR    = '\]';
-	    my $OpenParen  = '\(';   my $CloseParen  = '\)'; my $NonASCII   = '\x80-\xff';
-	    my $ctrl        = '\000-\037';                   my $CRlist     = '\012\015';
+        if (!$RFC822PAT) {
+            my $esc        = '\\\\'; my $Period      = '\.'; my $space      = '\040';
+            my $tab         = '\t';  my $OpenBR     = '\[';  my $CloseBR    = '\]';
+            my $OpenParen  = '\(';   my $CloseParen  = '\)'; my $NonASCII   = '\x80-\xff';
+            my $ctrl        = '\000-\037';                   my $CRlist     = '\012\015';
 
-	    my $qtext = qq/[^$esc$NonASCII$CRlist\"]/;
-	    my $dtext = qq/[^$esc$NonASCII$CRlist$OpenBR$CloseBR]/;
-	    my $quoted_pair = qq< $esc [^$NonASCII] >; # an escaped character
-	    my $ctext   = qq< [^$esc$NonASCII$CRlist()] >;
-	    my $Cnested = qq< $OpenParen $ctext* (?: $quoted_pair $ctext* )* $CloseParen >;
-	    my $comment = qq< $OpenParen $ctext* (?: (?: $quoted_pair | $Cnested ) $ctext* )* $CloseParen >;
-	    my $X = qq< [$space$tab]* (?: $comment [$space$tab]* )* >;
-	    my $atom_char  = qq/[^($space)<>\@,;:\".$esc$OpenBR$CloseBR$ctrl$NonASCII]/;
-	    my $atom = qq< $atom_char+ (?!$atom_char) >;
-	    my $quoted_str = qq< \" $qtext * (?: $quoted_pair $qtext * )* \" >;
-	    my $word = qq< (?: $atom | $quoted_str ) >;
-	    my $domain_ref  = $atom;
-	    my $domain_lit  = qq< $OpenBR (?: $dtext | $quoted_pair )* $CloseBR >;
-	    my $sub_domain  = qq< (?: $domain_ref | $domain_lit) $X >;
-	    my $domain = qq< $sub_domain (?: $Period $X $sub_domain)* >;
-	    my $route = qq< \@ $X $domain (?: , $X \@ $X $domain )* : $X >;
-	    my $local_part = qq< $word $X (?: $Period $X $word $X )* >;
-	    my $addr_spec  = qq< $local_part \@ $X $domain >;
-	    my $route_addr = qq[ < $X (?: $route )?  $addr_spec > ];
-	    my $phrase_ctrl = '\000-\010\012-\037'; # like ctrl, but without tab
-	    my $phrase_char = qq/[^()<>\@,;:\".$esc$OpenBR$CloseBR$NonASCII$phrase_ctrl]/; 
-	    my $phrase = qq< $word $phrase_char * (?: (?: $comment | $quoted_str ) $phrase_char * )* >;
-	    $RFC822PAT = qq< $X (?: $addr_spec | $phrase $route_addr) >;
-	}
+            my $qtext = qq/[^$esc$NonASCII$CRlist\"]/;
+            my $dtext = qq/[^$esc$NonASCII$CRlist$OpenBR$CloseBR]/;
+            my $quoted_pair = qq< $esc [^$NonASCII] >; # an escaped character
+            my $ctext   = qq< [^$esc$NonASCII$CRlist()] >;
+            my $Cnested = qq< $OpenParen $ctext* (?: $quoted_pair $ctext* )* $CloseParen >;
+            my $comment = qq< $OpenParen $ctext* (?: (?: $quoted_pair | $Cnested ) $ctext* )* $CloseParen >;
+            my $X = qq< [$space$tab]* (?: $comment [$space$tab]* )* >;
+            my $atom_char  = qq/[^($space)<>\@,;:\".$esc$OpenBR$CloseBR$ctrl$NonASCII]/;
+            my $atom = qq< $atom_char+ (?!$atom_char) >;
+            my $quoted_str = qq< \" $qtext * (?: $quoted_pair $qtext * )* \" >;
+            my $word = qq< (?: $atom | $quoted_str ) >;
+            my $domain_ref  = $atom;
+            my $domain_lit  = qq< $OpenBR (?: $dtext | $quoted_pair )* $CloseBR >;
+            my $sub_domain  = qq< (?: $domain_ref | $domain_lit) $X >;
+            my $domain = qq< $sub_domain (?: $Period $X $sub_domain)* >;
+            my $route = qq< \@ $X $domain (?: , $X \@ $X $domain )* : $X >;
+            my $local_part = qq< $word $X (?: $Period $X $word $X )* >;
+            my $addr_spec  = qq< $local_part \@ $X $domain >;
+            my $route_addr = qq[ < $X (?: $route )?  $addr_spec > ];
+            my $phrase_ctrl = '\000-\010\012-\037'; # like ctrl, but without tab
+            my $phrase_char = qq/[^()<>\@,;:\".$esc$OpenBR$CloseBR$NonASCII$phrase_ctrl]/;
+            my $phrase = qq< $word $phrase_char * (?: (?: $comment | $quoted_str ) $phrase_char * )* >;
+            $RFC822PAT = qq< $X (?: $addr_spec | $phrase $route_addr) >;
+        }
 
-	return scalar ($_[0] =~ /$RFC822PAT/ox);
+        return scalar ($_[0] =~ /$RFC822PAT/ox);
     }
 }
 
+sub _yn {
+    return $_[0] ? '[Y/n]: ' : '[y/N]';
+}
 
 1;
 
@@ -1469,21 +1558,19 @@ CPANPLUS::Configure::Setup - Configuration setup for CPAN++
 
 =head1 SYNOPSIS
 
-You will be automatically thrown to Setup when you install 
+You will be automatically thrown to Setup when you install
 CPANPLUS, or whenever your saved Config is corrupt.
 
 You can run Setup explicitly (which will replace your existing Config) with:
 
-    perl -MCPANPLUS::Configure::Setup -e 'CPANPLUS::Configure::Setup->init()'   
+    perl -MCPANPLUS::Configure::Setup -e 'CPANPLUS::Configure::Setup->init()'
 
 =head1 DESCRIPTION
 
 CPANPLUS::Configure::Setup prompts the user to enter information
-that will be used by CPANPLUS.   The text accompanying the questions
-
+that will be used by CPANPLUS.  The text accompanying the questions
 should be sufficient to guide the user through the configuration.
-
-The result of this inquiry is stored in Config.pm.   By default, this
+The result of this inquiry is stored in Config.pm.  By default, this
 information will be used by all CPANPLUS modules.  However, it is
 possible to change some configuration options at runtime with
 
