@@ -1,5 +1,5 @@
 # $File: //member/autrijus/cpanplus/dist/lib/CPANPLUS/Internals/Make.pm $
-# $Revision: #3 $ $Change: 3544 $ $DateTime: 2002/03/26 07:48:03 $
+# $Revision: #11 $ $Change: 3808 $ $DateTime: 2002/04/09 06:44:56 $
 
 #######################################################
 ###             CPANPLUS/Internals/Make.pm          ###
@@ -13,6 +13,7 @@ package CPANPLUS::Internals::Make;
 
 use strict;
 use File::Spec;
+use FileHandle;
 use CPANPLUS::Configure;
 use CPANPLUS::Error;
 use Cwd;
@@ -24,44 +25,18 @@ BEGIN {
     $VERSION    =   $CPANPLUS::Internals::VERSION;
 }
 
-sub _run {
-    my ($self, $cmd, $verbose) = @_;
-
-    $verbose = $self->{_conf}->get_conf('verbose')
-        unless defined $verbose;
-
-    return !system($cmd) if $verbose; # prints everything
-
-    # non-verbose mode: inhibit STDOUT
-    my $err  = $self->{_error};
-
-    local *SAVEOUT;
-
-    unless (open(SAVEOUT, ">&STDOUT")) {
-        $err->trap( error => "couldn't dup STDOUT: $!" );
-        return 0;
-    }
-
-    open STDOUT, '>'. File::Spec->devnull;
-
-    my $rv = system($cmd);
-
-    unless (open(STDOUT, ">&SAVEOUT")) {
-        $err->trap( error => "couldn't restore STDOUT: $!" );
-        return 0;
-    }
-
-    return !$rv;
-}
-
 sub _make {
-    my $self = shift;
-    my %args = @_;
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    my $self    = shift;
+    my %args    = @_;
+    my $conf    = $self->{_conf};
+    my $err     = $self->{_error};
+    my $modtree = $self->_module_tree;
 
-    my $dir = $args{'dir'};
-    my $data = $args{'module'};
+    my $dir    = $args{'dir'};
+    my $data   = $args{'module'};
+
+    ### make target: may also be 'makefile', 'make', 'test' or 'skiptest'
+    my $target = lc($args{'target'}) || 'install';
 
     ### test for Backend->flush
     ###print "this is todo: ", Dumper $self->{_todo};
@@ -76,18 +51,22 @@ sub _make {
     ### perhaps we shouldn't store the version of perl in config, but just use the one
     ### we were invoked with ($^X) unless the user specifies one specifically...
     ### i think that would make a few users happier -kane
-    my $perl   = $args{'perl'}              || $^X;
-    my $force  = $args{'force'}             || $conf->get_conf('force');
-    my $flag   = $args{'makeflags'}         || $conf->get_conf('makeflags');
-    my $make   = $args{'make'}              || $conf->_get_build('make');
-    my %mmflags= %{$args{'makemakerflags'}  || $conf->get_conf('makemakerflags')};
+    my ($perl, $force, $make, $report, $mmflags, $makeflags)
+        = @args{qw|perl force make cpantest makemakerflags makeflags|};
+
+    ### fill in the defaults; checks for definedness, not truth value. -autrijus
+    $perl      = $^X                               unless defined $perl;
+    $force     = $conf->get_conf('force')          unless defined $force;
+    $make      = $conf->_get_build('make')         unless defined $make;
+    $report    = $conf->get_conf('cpantest')       unless defined $report;
+    $mmflags   = $conf->get_conf('makemakerflags') unless defined $mmflags;
+    $makeflags = $conf->get_conf('makeflags')      unless defined $makeflags;
 
     my $verbose = $conf->get_conf('verbose');
+    my $captured; # capture buffer of _run
 
     ### try to install the module ###
 
-    #unless ( open $fh, qq($perl Makefile.PL PREFIX=$target |) ) {
-
     ### 'perl Makefile.PL' ###
     ### check if the makefile.pl exists
     unless ( -e 'Makefile.PL' ) {
@@ -105,85 +84,54 @@ sub _make {
             $err->trap(
                 error => "Could not generate Makefile.PL - Aborting"
             );
+            $self->_restore_startdir;
             return 0;
         }
     }
-    ### We HAVE to use system() here so as to allow for interactive
-    ### makefiles. Here's the rundown of conciderations:
-    ###
-    ### cmd         verbose ctrl   rv?      interactive?
-    ### system          n           y           y
-    ### backticks       y           y           n
-    ### exec            y           n           y
-    ### open            y           y           n
-    ###
-    ### seeing we NEED a return value (to do error checks),
+
     ### we can only use open, backticks or system. and only the latter allows
     ### interactive mode. so we're screwed =/ -Kane
+    ### not really; there's open3 -- see the _run() method. -autrijus
 
     PERL_MAKEFILE: {
-        ### we can check for a 'Makefile' but that might be have produced a 
+        ### we can check for a 'Makefile' but that might be have produced a
         ### Makefile that CAME from a different version of perl
         ### thus screwing everything up.
         ### i suggest uncommenting this once we have a way to pass make options like 'clean'
-        
+
         #if ( -e 'Makefile' && !$force ) {
         #    $err->inform( msg => qq[Makefile already exists, not running 'perl Makefile.PL' again, unless you force!], quiet => !$verbose );
         #    last PERL_MAKEFILE;
         #}
 
-        my @args = map {
-            (defined $mmflags{$_}) ? "$_=$mmflags{$_}" : $_
-        } sort keys %mmflags;
+        my @args = @{$self->_flags_arrayref($mmflags)};
 
-        unless( $self->_run( "$perl Makefile.PL @args", 1 ) ) { # always verbose
+        unless( $self->_run(
+            command => [$perl, 'Makefile.PL', @args],
+            buffer  => \$captured,
+            verbose => 1
+        ) ) {
             ### store it if a module failed to install for some reason ###
             $self->{_todo}->{failed}->{ $data->{module} } = 1;
 
             $err->trap( error => "BUILDING MAKEFILE failed! - $!" );
+            $self->_send_report( module => $data, buffer => $captured) if $report;
+            $self->_restore_startdir;
             return 0;
         }
+
+        if ($target eq 'makefile') {
+            $self->_restore_startdir;
+            return 1;
+        }
     }
 
-#    {
-#        local $?;   # be sure it's undef
-#        my @output = `$perl Makefile.PL PREFIX=$target`;
-#        unless ( $? ) {
-#            for (@output) { chomp; $err->inform( msg => $_, quiet => !$verbose ) }
-#        } else {
-#            $err->trap( error => "BUILDING MAKEFILE failed! - $?" );
-#            return 0;
-#        }
-#    }
-
-#    {   my $fh;
-#        open $fh, "$perl Makefile.PL PREFIX=$target |" or
-#            (   $err->trap( error => "BUILDING MAKEFILE failed! - $?" ),
-#                return 0
-#            );
-#
-#        while (<$fh>) { chomp; $err->inform( msg => $_, quiet => !$verbose ) }
-#        close $fh;
-#    }
-
-    ### the way cpan.pm does it ###
-    ### definate no-no
-#    $system = "$perl $switch Makefile.PL $CPAN::Config->{makepl_arg}";
-#
-#    if (defined($pid = fork)) {
-#        if ($pid) { #parent
-#            # wait;
-#            waitpid $pid, 0;
-#        } else {    #child
-#            exec $system;
-#        }
-#    }
 
     ### this is where we find out what prereqs this module has,
     ### and install them accordingly.
     ### this probably needs some tidying up ###
 
-   
+
     my $prereq = $self->_find_prereq( dir => $dir );
 
     ### check if the prereq this module wants is something we already tried to install
@@ -210,7 +158,10 @@ sub _make {
             }
         }
 
-        return 0 if $flag;
+        if ($flag) {
+            $self->_restore_startdir;
+            return 0;
+        }
     }
 
     ### if we're not allowed to follow prereqs, and there are some
@@ -225,13 +176,13 @@ sub _make {
     ### of the array ref as we go.
     ### then, check the array ref for what modules are installed already
     ### yes, i know, messy... -kane
-    
+
     ### we now also have module objects, so i suppose we should start storing this information
     ### THERE instead? definately a TODO!
-    
+
     my $must_install;
     my %list;
-     
+
     for my $mod (keys %$prereq) {
 
         ### check if the module (with specified version) is installed yet
@@ -251,18 +202,13 @@ sub _make {
                     msg     => "Prereqs are found, but not allowed to install! Returning list of prereqs",
                     quiet   => !$verbose
                 );
+
+                $self->_restore_startdir;
                 return \@{[ keys %list ]};
             }
 
-            ### check if any of the prereqs we're about to install wants us to get
-            ### a newer version of perl... if so, skip, we dont want to upgrade perl
-            if ($mod =~ /^base$/i or $self->{_modtree}->{$mod}->{package} =~ /^perl\W/i ) {
-                $err->inform(
-                    msg => "The module you're trying to install wants to upgrade your version of perl" .
-                            " but you probably dont want that, so we're skipping",
-                    quiet => !$verbose
-                );
-
+            unless ( keys(%{$modtree->{$mod}}) ) {
+                $err->trap( error => "No such module: $mod, cannot satisfy dependency" );
                 next;
             }
 
@@ -286,22 +232,23 @@ sub _make {
 
 
     ### see if we have anything to install, if so, we'll need to exit this make, and install
-    ### the prereqs first.    
+    ### the prereqs first.
     if (%list) {
 
-        ### store this dir, we'll have to finish the make here later.
-        unshift @{$self->{_todo}->{make}}, $dir;
+        ### store this dir and modname, we'll have to finish the make here later.
+        unshift @{$self->{_todo}->{make}}, \%args;
 
         ### enqueue this modules prereqs ###
         unshift @{$self->{_todo}->{install}}, [ map {
-            $self->{_modtree}->{$_}
+            $modtree->{$_}
         } keys %list ];
-    
+
         while (my $mod_ref = shift @{$self->{_todo}->{install}} ) {
             $self->_install_module( modules => $mod_ref );
         }
-    
+
     } else {
+        my @args = @{$self->_flags_arrayref($makeflags)};
 
         INSTALL: {
             ### ok, so we have no prereqs to take care of, let's go on with installing ###
@@ -311,63 +258,132 @@ sub _make {
                 ### Makefile that CAME from a different version of perl
                 ### thus screwing everything up.
                 ### i suggest uncommenting this once we have a way to pass make options like 'clean'
-                
+
                 #if ( -d 'blib' && !$force ) {
                 #    $err->inform( msg => qq[Already ran 'make' for this module. Not running again unless you force!], quiet => !$verbose );
                 #    last MAKE;
                 #}
-    
-                unless ( $self->_run("$make $flag") ) {
+
+                unless ( $self->_run(
+                    command => [$make, @args],
+                    buffer  => \$captured,
+                ) ) {
                     ### store it if a module failed to install for some reason ###
                     $self->{_todo}->{failed}->{ $data->{module} } = 1;
-    
+
                     $err->trap( error => "MAKE failed! - $!" );
+                    $self->_send_report( module => $data, buffer => $captured) if $report;
+                    $self->_restore_startdir;
                     return 0;
                 }
+
+                last INSTALL if $target eq 'make';
             }
-    
-            {
-                unless ( $self->_run("$make $flag test", 1) ) { # always verbose
+
+            if ($target ne 'skiptest') {
+                unless ( $self->_run(
+                    command => [$make, @args, 'test'],
+                    buffer  => \$captured,
+                    verbose => 1
+                ) ) {
                     ### store it if a module failed to install for some reason ###
                     $self->{_todo}->{failed}->{ $data->{module} } = 1;
-    
+
                     $err->trap( error => "MAKE TEST failed! - $!" );
+                    $self->_send_report( module => $data, buffer => $captured) if $report;
+                    $self->_restore_startdir;
                     return 0;
                 }
+
+                $self->_send_report( module => $data, buffer => $captured) if $report;
+                last INSTALL if $target eq 'test';
             }
-    
+
             {
-                unless ( $self->_run("$make $flag install") ) {
+                unless ( $self->_run(
+                    command => [$make, @args, 'install'],
+                ) ) {
                     ### store it if a module failed to install for some reason ###
                     $self->{_todo}->{failed}->{ $data->{module} } = 1;
-    
+
                     $err->trap( error => "MAKE INSTALL failed! - $!" );
+                    $self->_restore_startdir;
                     return 0;
                 }
             }
-    
-            ### chdir back to the dir where the script is running in ###
-            chdir $conf->_get_build('startdir') or
-                $err->inform(
-                    msg     => "Invalid start dir!",
-                    quiet   => !$verbose
-                );
-    
-            ### nothing went wrong, but we DID install... mark that as well ###
-            $self->{_todo}->{failed}->{ $data->{module} } = 0;
-    
-            return 1;
+
         }
+
+        $self->_restore_startdir;
+
+        ### nothing went wrong, but we DID install... mark that as well ###
+        $self->{_todo}->{failed}->{ $data->{module} } = 0;
+
+        return 1;
     }
- 
+
     ### if we still have modules left to do in our _tomake list, this is the time to do it!
-    if ( length @{$self->{_todo}->{make}} ) { $self->_make( dir => shift @{$self->{_todo}->{make}} ) }
+    if ( @{$self->{_todo}->{make}} ) {
+        $self->_make( %{ shift @{$self->{_todo}->{make}} } );
+    }
 
     ### indicate success ###
     return 1;
 
 } #_make
 
+
+### convert scalar 'var=val' or orrayref flags into a hashref
+sub _flags_hashref {
+    my ($self, $flags) = @_;
+
+    ### first, join arrayref flags (like ['A=B C=D', 'E=F']) together
+    $flags = join(' ', @{$flags}) if UNIVERSAL::isa($flags, 'ARRAY');
+
+    ### next, split scalars into a hashref and hand it to the caller
+    $flags = {
+        map { /=/ ? split('=', $_, 2) : ($_ => undef) }
+            $flags =~ m/\s*((?:[^\s=]+=)?(?:"[^"]+"|'[^']+'|[^\s]+))/g
+    } unless UNIVERSAL::isa($flags, 'HASH');
+
+    return $flags;
+}
+
+
+### convert scalar or hashref flags into an arrayref
+sub _flags_arrayref {
+    my ($self, $flags) = @_;
+
+    ### first, split scalar flags into hashref
+    $flags = $self->_flags_hashref($flags) unless ref($flags);
+
+    ### next, parse the hashref to an array and return it
+    $flags = [ map {
+        (defined $flags->{$_}) ? "$_=$flags->{$_}" : $_
+    } sort keys %{$flags} ] if UNIVERSAL::isa($flags, 'HASH');
+
+    return $flags;
+}
+
+
+### chdir back to the dir where the script is running in ###
+sub _restore_startdir {
+    my $self = shift;
+    my $conf = $self->{_conf};
+    my $err  = $self->{_error};
+    my $verbose = $conf->get_conf('verbose');
+
+    return 1 if chdir($conf->_get_build('startdir'));
+
+    $err->inform(
+        msg     => "Invalid start dir!",
+        quiet   => !$verbose
+    );
+
+    return 0;
+}
+
+
 ### sub to generate a Makefile.PL in case the module didn't ship with one
 sub _make_makefile {
     my $self = shift;
@@ -440,4 +456,167 @@ sub _find_prereq {
     return \%p;
 }
 
+
+### Execute a command: $cmd may be a scalar or an arrayref of cmd and args
+### $bufout is an scalar ref to store outputs, $verbose can override conf
+sub _run {
+    my ($self, %args) = @_;
+    my ($cmd, $buffer, $verbose) = @args{qw|command buffer verbose|};
+    my $err = $self->{_error};
+    my ($buferr, $bufout);
+
+    $$buffer = '';
+    $verbose = $self->{_conf}->get_conf('verbose')
+        unless defined $verbose;
+
+    ### STDOUT message handler
+    my $_out_handler = sub {
+        my $buf = shift; print STDOUT $buf if $verbose;
+        $$buffer .= $buf; $bufout .= $buf;
+        $err->inform( msg => $1, quiet => 1 ) while $bufout =~ s/(.*)\n//;
+    };
+
+    ### STDERR message handler
+    my $_err_handler = sub {
+        my $buf = shift; print STDERR $buf if $verbose;
+        $$buffer .= $buf; $buferr .= $buf;
+        $err->trap( error => $1, quiet => 1 ) while $buferr =~ s/(.*)\n//;
+    };
+
+    my @cmd = ref($cmd) ? grep(length, @{$cmd}) : $cmd;
+    my $is_win98 = ($^O eq 'MSWin32' and !Win32::IsWinNT());
+
+    ### inform the user. note that we don't want to used mangled $verbose.
+    $err->inform(
+        msg   => "Running [@cmd]...",
+        quiet => !$self->{_conf}->get_conf('verbose'),
+    );
+
+    ### First, we prefer Barrie Slaymaker's wonderful IPC::Run module.
+    if (!$is_win98 and $self->_can_use(
+        modules  => { 'IPC::Run' => '0.55' },
+        complain => ($^O eq 'MSWin32'),
+    ) ) {
+        IPC::Run::run(\@cmd, \*STDIN, $_out_handler, $_err_handler);
+    }
+
+    ### Next, IPC::Open3 is know to fail on Win32, but works on Un*x.
+    elsif ($^O ne 'MSWin32' and $self->_can_use(
+        modules => { map { $_ => '0.0' } qw|IPC::Open3 IO::Select Symbol| },
+    ) ) {
+        $self->_open3_run(\@cmd, $_out_handler, $_err_handler);
+    }
+
+    ### Abandon all hope; falls back to simple system() on verbose calls.
+    elsif ($verbose) {
+        system(@cmd);
+    }
+
+    ### Non-verbose system() needs to have STDOUT and STDERR muted.
+    else {
+        local *SAVEOUT; local *SAVEERR;
+
+        open(SAVEOUT, ">&STDOUT")
+            or ($err->trap( error => "couldn't dup STDOUT: $!" ), return);
+        open(STDOUT, ">".File::Spec->devnull)
+            or ($err->trap( error => "couldn't reopen STDOUT: $!" ), return);
+
+        open(SAVEERR, ">&STDERR")
+            or ($err->trap( error => "couldn't dup STDERR: $!" ), return);
+        open(STDERR, ">".File::Spec->devnull)
+            or ($err->trap( error => "couldn't reopen STDERR: $!" ), return);
+
+        system(@cmd);
+
+        open(STDOUT, ">&SAVEOUT")
+            or ($err->trap( error => "couldn't restore STDOUT: $!" ), return);
+        open(STDERR, ">&SAVEERR")
+            or ($err->trap( error => "couldn't restore STDERR: $!" ), return);
+    }
+
+    $_out_handler->("\n") if length $bufout;
+    $_err_handler->("\n") if length $buferr;
+
+    return !$?;
+}
+
+
+### IPC::Run::run emulator, using IPC::Open3.
+sub _open3_run {
+    my ($self, $cmdref, $_out_handler, $_err_handler) = @_;
+    my $err = $self->{_error};
+    my @cmd = @{$cmdref};
+
+    ### Following code are adapted from Friar 'abstracts' in the
+    ### Perl Monastery (http://www.perlmonks.org/index.pl?node_id=151886).
+
+    local *SAVEIN;
+
+    ### saving away STDIN first, because it will be closed by open3
+    unless (open(SAVEIN, ">&STDIN")) {
+        $err->trap( error => "couldn't dup STDIN: $!" );
+        return 0;
+    }
+
+    my ($outfh, $errfh); # open3 handles
+
+    my $pid = eval {
+        IPC::Open3::open3(
+            '<&STDIN', # may also be \*STDIN according to Barrie
+            $outfh = Symbol::gensym(),
+            $errfh = Symbol::gensym(),
+            @cmd,
+        )
+    };
+
+    if ($@) {
+        $err->trap( error => "couldn't spawn process: $@" );
+        return;
+    }
+
+    my $sel = IO::Select->new; # create a select object
+    $sel->add($outfh, $errfh); # and add the fhs
+
+    STDOUT->autoflush(1); STDERR->autoflush(1);
+    $outfh->autoflush(1) if UNIVERSAL::can($outfh, 'autoflush');
+    $errfh->autoflush(1) if UNIVERSAL::can($errfh, 'autoflush');
+
+    while (my @ready = $sel->can_read) {
+        foreach my $fh (@ready) { # loop through buffered handles
+            # read up to 4096 bytes from this fh.
+            my $len = sysread $fh, my($buf), 4096;
+
+            if (not defined $len){
+                # There was an error reading
+                $err->trap( error => "Error from child: $!" );
+                return;
+            }
+            elsif ($len == 0){
+                $sel->remove($fh); # finished reading
+                next;
+            }
+            elsif ($fh == $outfh) {
+                $_out_handler->($buf);
+            } elsif ($fh == $errfh) {
+                $_err_handler->($buf);
+            } else {
+                $err->trap( error => "IO::Select error" );
+                return;
+            }
+        }
+    }
+
+    waitpid $pid, 0; # wait for it to die
+
+    ### restore STDIN
+    local $^W; # quell 'Filehandle STDIN opened only for output' warnings
+    unless (open(STDIN, ">&SAVEIN")) {
+        $err->trap( error => "couldn't restore STDIN: $!" );
+        return 0;
+    }
+
+    return 1;
+}
+
+
 1;
